

classdef DesignSemiCircle <Design 
    %D=DesignSemiCircle creates an instance of a design D(x) given by:
    %
    %       D(x)={ y in R^2 with:
    %               (y1-x1)^2 + (y2-x2)^2 - x3^2<= 0
    %               (y1-x1)*x4/sqrt(x4^2+x5^2)+ (y2-x2)*x5/sqrt(x4^2+x5^2)<= 0}
    %       
    %       Vol(x)=pi/2*x3^2
    %
    %       x in X = { x in R^5 with
    %               10^-6 <= x3
    %               -x4^2-x5^2+1 <= 0}
    
    properties (GetAccess=protected)
        
        
        dim=2;                                  
        nrOfParams=5                            
        nrOfDesignConstrs= 2;                    
        initValsParams=[0,0,1,1,0];         
        
        linEqParamConstrs =[];
        linIneqParamConstrs=[];  
        boundsOnParams=[-inf,-inf,1e-6,-inf,-inf;inf,inf,inf,inf,inf]
        nrOfNonlinParamEqConstrs=0;              
        nrOfNonlinParamIneqConstrs =1;           

       
    end

    
    methods
        %volume
        function v=vol(~,x)                     
            v=pi/2*x(3)^2;
        end
        function grad_v=gradVol(~,x)           
            grad_v=[0;0;pi*x(3);0;0];
        end
        
        %design functions
        function u=evalNonlinDescrFuns(~,x,y)                
            u=[(y(1)-x(1))^2+(y(2)-x(2))^2-x(3)^2,...
                (y(1)-x(1))*x(4)/sqrt(x(4)^2+x(5)^2)+(y(2)-x(2))*x(5)/sqrt(x(4)^2+x(5)^2)];
        end    
        function Dy_u=DyNonlinDescrFuns(~,x,y)               
            Dy_u = zeros(2,2);
            Dy_u(1,:) = [2*(y(1)-x(1)), 2*(y(2)-x(2))];
            Dy_u(2,:) = x(4:5)/sqrt(x(4)^2+x(5)^2);
        end
        function Dx_u=DxNonlinDescrFuns(~,x,y)
            Dx_u = zeros(2,5);
            Dx_u(1,1:3) = [-2*(y(1)-x(1)), -2*(y(2)-x(2)), -2*x(3)];
            l = sqrt(x(4)^2+x(5)^2);
            Dx_u(2,1:2) = -x(4:5)/l;
            t1 = x(5)*(x(1)-y(1))+x(4)*(y(2)-x(2));
            t2 = t1/l^3;
            Dx_u(2,4) = -x(5)*t2;
            Dx_u(2,5) = x(4)*t2;   
        end
        %Interior point
        function y=getInteriorPoint(~,x)                    
            y=x(1:2)-x(3)/4*x(4:5);
        end
        function Dx_y=DxInnerPoint(~,x)
            Dx_y=[eye(2),-0.25*x(4:5)',-x(3)/4*eye(2)];
        end
        
        %bounding box
        function [lb,ub]=getBoundingBox(~,x)            
            lb=[x(1)-x(3),x(2)-x(3)];
            ub=[x(1)+x(3),x(2)+x(3)];
        end

        %non-linear constraints on parameters 
        function k=evalNonlinIneqConstrsOnParams(~,x)        
            k=-x(4)^2-x(5)^2+1;
        end    
 
        function Dx_k=DxNonlinIneqConstrsOnParams(~,x)       
            Dx_k=[0,0,0,-2*x(4),-2*x(5)];
        end 
        
        function [lb,ub]=getBox(~)
            lb=[0,-0.5];
            ub=[1,0.5];
        end
        
    end
    
    %transformation ata
    methods
        function y=evalTrafoFun(obj,x,z)
             o = ones(size(z,1),1);
            ang = atan2(-x(5),-x(4));
  
            y = [x(1)*o + x(3)*z(:,1).*cos(ang*o+pi*z(:,2)), ...
            x(2)*o + x(3)*z(:,1).*sin(ang*o+pi*z(:,2))];
        end
        function z=evalInvTrafoFun(obj,x,y)
            m=size(y,1);
            o = ones(m,1);
            
            ang = atan2(-x(5),-x(4));
            cos_ang = cos(ang);
            sin_ang = sin(ang);

            R = [ cos_ang, sin_ang; % mathematically negative
                 -sin_ang, cos_ang];

            y1ss = 1/x(3)*(y(:,1)-x(1)*o);
            y2ss = 1/x(3)*(y(:,2)-x(2)*o);

            y1ssr = R(1,1)*y1ss + R(1,2)*y2ss;
            y2ssr = R(2,1)*y1ss + R(2,2)*y2ss;

            [THETA,RHO] = cart2pol(y1ssr,y2ssr);
            z = [RHO,THETA/pi];
        end
        function Dx_y=DxTrafoFun(obj,x,z)
            m = size(z,1);
              o = ones(m,1);

              ang = atan2(-x(5),-x(4));

              cos_angs = cos(ang*o+pi*z(:,2));
              sin_angs = sin(ang*o+pi*z(:,2));

              z1_cos_angs = z(:,1).*cos_angs;
              z1_sin_angs = z(:,1).*sin_angs;

              d = cos(ang)^2;

              term1 = x(3)/x(4)*z1_cos_angs*d;
              term2 = x(3)/x(4)*z1_sin_angs*d;

              Dx_y = zeros(2,5,m);
              Dx_y(1,1,:) = o;
              Dx_y(2,2,:) = o;
              Dx_y(1,3,:) = z1_cos_angs;
              Dx_y(2,3,:) = z1_sin_angs;
              Dx_y(1,4,:) = x(5)/x(4)*term2;
              Dx_y(1,5,:) = -term2;
              Dx_y(2,4,:) = -x(5)/x(4)*term1;
              Dx_y(2,5,:) = term1;
        end
    end
    
    methods
        function []=plotDesign(obj,x)
                        ang = atan2(-x(5),-x(4));
            t = (0:pi/180:pi)-pi/2;
            o = ones(size(t));

            y1 = x(1)*o+x(3)*cos(ang+t);
            y2 = x(2)*o+x(3)*sin(ang+t);

            fill(y1,y2,'green');
            hold on;
        end
    end
    
end

