classdef Problem19 < ImplicitGSIPProblem
    % p=Problem19 creates an instance of the GSIP problem:
    %
    %       min   x1^2+x2^2+x3^2
    %
    %       s.t.  x1+x2*exp(x3*y)+exp(2y)-2sin(4y) <=0 for all y in Y(x)
    %             -5 <= x1 <= 5
    %             -1+10^-6 <= x2 <= 5
    %             -5 <= x3 <= 5
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               2y<= x2+1
    %               0 <= y <= 1}
 
    % One constraint is added, such that Y(x) doesn't become empty  
    properties(GetAccess=protected)

        
        nrOfUppLevVars=3;                         
        boundsOnUppLevVars=[-5,-5,-5;5,5,5];       
        nrOfSemiInfConstrs=1;                      
        initValsOfUppLevVars=[0,0,0];             
        
        linEqsOnUppLevVars=[];                    
        linIneqsOnUppLevVars=[0,-1,0,1-1.e-6];         
        nrOfNonlinEqsUppLev=0;               
        nrOfNonlinIneqsUppLev=0;             
        
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=0;                       
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end

    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=x(1)^2+x(2)^2+x(3)^2;
        end
        function grad_f=gradObjFun(obj,x)                   
            grad_f=2*x';
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)      
                                                         
            g=x(1)+x(2)*exp(x(3)*y)+exp(2*y)-2*sin(4*y);            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)     
            n=size(y,1);
            Dx_g=[ones(n,1),exp(x(3)*y),x(2)*y.*exp(x(3)*y)];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=x(2)*x(3)*exp(x(3)*y)+2*exp(2*y)-8*cos(4*y);                
        end

            
        function y=getInitValsLowLevVars(obj,~,x)           
            [lb,ub] = obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)    
             lb=0;
             ub=1;
        end
                        
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,k,x)
            A=[2];
            b=[x(2)+1];
        end
        
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
    end
    methods(Access=private)       
        
        function [lb,ub] = getInterval(obj,x)           
            lb=0;
            ub=(x(2)+1)/2;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            Dx_lb=[0,0,0];
            Dx_ub=[0,0.5,0];
        end

    end
    
end